define(
[
  'dojo/_base/declare',
  'dojo/on',
  'dojo/date/locale',
  'jimu/BaseWidget',
  'esri/dijit/Directions',
  'esri/dijit/analysis/PlanRoutes',
  'esri/layers/FeatureLayer',
  'esri/geometry/webMercatorUtils',
  './_NARouteSharing'
],
  function(declare, on, locale, BaseWidget, Directions, PlanRoutes, FeatureLayer, webMercatorUtils, _NARouteSharing) {

    return declare([BaseWidget], {

      baseClass: 'jimu-widget-pizzalb',
      name: 'PizzaLB',

      _dw: null,
      _pr: null,
      _depotsLayer: null,
      _customersLayer: null,

      //keeping track of route which might
      //be modified inside the Directions widget
      _editedRoute: null,
      _routeModified: false,

      _naRouteSharing: null,

      //main initialization logic
      startup: function() {
        window.pz = this;
        this.inherited(arguments);

        this._naRouteSharing = new _NARouteSharing(portalUrl, this.map.spatialReference);

        this._depotsLayer = this.map.getLayer(this.map.graphicsLayerIds[0]);
        this._customersLayer = this.map.getLayer(this.map.graphicsLayerIds[1]);

        //creating Directions widget
        this._dw = new Directions({
          map: this.map,
          portalUrl: "http://www.arcgis.com",
          mapClickActive: true,
          canModifyStops: false,

          showSaveButton: true
        }, "dirContainer");
        window.dw = this._dw;
        this._dw.startup();

        //Directions widget events
        on(this._dw, "route-item-updated", function(){
          //route layer was saved again, means we will need to update the geometry
          //once Directions widget is closed
          this._routeModified = true;
        }.bind(this));
        dojo.byId("dir").style.display = "none";

        //Email Route button
        var bcont = document.createElement("div"),
          btn = document.createElement("button"),
          dirHeader = document.getElementById("dir");
        bcont.style = "padding-left: 134px;";
        btn.innerHTML = "Email the route";
        btn.onclick = function(){
          var nav = "arcgis-navigator://";
          wgsStops = dw.stops.map(function(s, i){
            var g = webMercatorUtils.webMercatorToGeographic(s.feature.geometry);
            nav +=
              (!i? "?" : "&") +
              "stop=" + g.y + "," + g.x +
              "&stopname=" + s.name;
          });
          var
            p = this._naRouteSharing.portal,
            u = "https://" + (p.isPortal ? p.portalHostname : p.urlKey + "." + p.customBaseUrl),
            w = window.open("mailto:?subject=" + dw.routeLayer.title + "&body=" + encodeURIComponent(
              u + "/home/item.html?id=" + dw.routeLayer.itemId
            ).replace(/%20/g, "+"));
          setTimeout(function(){w.close();}, 3000);
          this._naRouteSharing.putNavigatorLinkIntoItemDescription(
            dw.routeLayer.itemId,
            dw._folderSelector.getValue(),
            //encodeURIComponent(nav).replace(/%20/g, "+")
            nav
          ).then(
            function(){console.log("updated.")},
            function(err){console.warn("Failed to update description. " + err)}
          );
        }.bind(this);
        bcont.appendChild(btn);
        dirHeader.appendChild(bcont);

        //creating Plan Routes widget
        var _15min = 15 * 60 * 1000,
          now = new Date();
        this._pr = new PlanRoutes({
          map: this.map,
          portalUrl: "http://www.arcgis.com",

          stopsLayer: this._customersLayer,
          startLayer: this._depotsLayer,
          stopsLayers: [this._customersLayer, this._depotsLayer],
          featureLayers: [this._customersLayer, this._depotsLayer],

          routeStartTime: Math.ceil(now.getTime() / _15min + 1) * _15min, //next 15 minutes departure
          outputLayerName: "Route to Customers - " +
          locale.format(new Date(), {
            selector: "date", datePattern: "MMMM d, h.m a"
          }),
          routeCount: 5,
          maxStopsPerRoute: 10,
          stopServiceTime: 300,
          maxRouteTime: 120,

          showSelectFolder: true,
          showOutputType: true,
          showHelp: false,
          showCredits: false,
          showChooseExtent: false,

          includeRouteLayers: true
        }, "planR");
        this._pr.startup();

        //event handling of the analysis widget
        var resetTool = function (status) {
          if (status === "error") {
            this._pr.set("disableRunAnalysis", false);
          }
          //Analysis started lets clean up any old results
          //Hide the legend, remove the results layer
          var layer = this.map.getLayer("resultLayer");
          if (layer) {
            this.map.removeLayer(layer);
          }
        }.bind(this);

        on(this._pr, "job-fail", function (params) {
          alert((params || {message: ""}).message);
          resetTool("error");
        });
        on(this._pr, "job-status", function (status) {
          if (status.jobStatus === 'esriJobFailed') {
            alert("Job Failed: " + status.messages[0].description);
            resetTool("error");
          }
        });
        on(this._pr, "job-cancel", function () {
          resetTool("error");
        });
        on(this._pr, "job-submit", function (result) {
          resetTool("submit");
        });

        on(this._pr, "job-result", function (res) {
          //add the results to the map and display the legend.
          if (res.value) {
            //var template = new InfoTemplate("Results", "${*}");
            var resultLayer = new FeatureLayer(res.value.url || res.value, {
              //infoTemplate: template,
              outFields: ["*"],
              opacity: 0.7,
              id: "resultLayer",
              mode: FeatureLayer.MODE_SNAPSHOT
            });

            //user clicks a route feature
            on(resultLayer, "click", function (e) {
              var rid = e.graphic && e.graphic.attributes.RouteLayerItemID;
              if (rid) {
                //hiding PlanRoutes panel and showing the Directions Widget panel
                dojo.byId("planR").style.display = "none";
                dojo.byId("dir").style.display = "block";

                this._updateGeometry();

                //loading route layer into Directions Widget
                this._dw.loadRoute(rid);

                //keeping track of the original feature user edits
                this._editedRoute = e.graphic;
              }
            }.bind(this));

            //making default route symbols a bit thicker
            on(resultLayer, "load", function() {
              var symbols = resultLayer.renderer && resultLayer.renderer.infos || [];
              for (var i = 0; i < symbols.length; i++) {
                symbols[i].symbol.width = 5;
              }
            });
            this.map.addLayer(resultLayer, 0);
          }

        }.bind(this));
      },

      //adding new customers coming as point graphics array
      addCustomer: function(pointGraphics){
        this._customersLayer.applyEdits(pointGraphics).then(this._customersLayer.refresh);
      },

      _handleCloseDW: function(){
        //hiding Directions widget and showing the Plan Routes widget again
        dojo.byId("dir").style.display = "none";
        dojo.byId("planR").style.display = "block";

        this._updateGeometry();

        //resetting state of the Directions widget
        this._dw.reset();
      },

      _updateGeometry: function(){
        //updating display with the modified route geometry
        if (this._routeModified) {
          this._editedRoute.setGeometry(this._dw.directions.mergedGeometry);
          this._editedRoute = null;
          this._routeModified = false;
        }
      }
    });
  });